%% Ptychographic reconstruction of attosecond pulses
% Thomas Feurer, Matteo Lucchini
% November 2015
% for details please contact:
% thomas.feurer@iap.unibe.ch
% mlucchini@phys.ethz.ch

%% console
% choice of what to reconstruct.
% if = 1: SAP with chirp
%    = 2: APT with chirp
%    = 3: SAP with satellite
%    = 4: SAP sparse sampling
choice = 4;

noi = 500;           % number of iterations
beta_R = 0.35;       % reconstruction constant
beta_P = 0.10;       % reconstruction constant
verb = 20;           % verb = # iterations when the results will be displayed

switch choice
    case 1
        load(fullfile('data','SAP1.mat'))
        A0 = 6; % starting strength of the gate
    case 2
        load(fullfile('data','APT1.mat'))
        A0 = 1; % starting strength of the gate
    case 3
        load(fullfile('data','SAP2.mat'))
        A0 = 6; % starting strength of the gate
    case 4
        load(fullfile('data','SAP3.mat'))
        A0 = 6; % starting strength of the gate
end

fn = length(f);
taun = length(tau);
w = 2*pi*f;

%% plotting simulated data to be reconstructed

figure(1)
% spectrogram
subplot(2,2,1)
pcolor(tau,W,Isim)
shading flat
xlabel('Delay (fs)','Fontsize',16)
ylabel('Energy','Fontsize',16)
set(gcf,'Color',[1 1 1])
set(gca,'Fontsize',14)
ylim(Yrange)
colormap(gca,tmap)

% IR
subplot(2,2,2)
plot(t,IR_abs,'o--r')
xlabel('Time (fs)','Fontsize',16)
ylabel('Norm. Ampl','Fontsize',16)
xlim([-10 10])
legend('E_{IR}(t)')
set(gca,'Fontsize',14)

% XUV amplitude
Rt_abs = abs(ht);
Rt_ph = angle(ht);
subplot(2,2,3)
plot(t,Rt_abs,'--ok')
xlabel('Time (fs)','Fontsize',16)
ylabel('Norm. Ampl','Fontsize',16)
xlim(Xrange)
legend('E_{XUV}(t)')
set(gca,'Fontsize',14)

% XUV phase
Rt_ph2 = Rt_ph;
Rt_ph2(Rt_abs<0.01) = NaN;
Rt_ph2 = unwrap(Rt_ph2);
Rt_ph2 = Rt_ph2-min(Rt_ph2);
subplot(2,2,4)
plot(t,Rt_ph2,'--ob')
xlabel('Time (fs)','Fontsize',16)
ylabel('Phase (rad)','Fontsize',16)
xlim(Xrange)
legend('E_{XUV}(t)')
set(gca,'Fontsize',14)

%% reconstruction
% plotting the progress
scrsz = get(0,'ScreenSize');
h0 = figure('NumberTitle','off','Position',[1 1 0.9*scrsz(3) 0.9*scrsz(4)]);
m2 = uimenu('Parent',h0,'Label','?','Callback',' figure(''Name'',''FROG-CRAB'',''MenuBar'',''none'',''NumberTitle'',''off'');iptsetpref(''ImshowBorder'', ''tight'');imshow(''boda.gif'');');

% starting spectrogram
subplot(3,2,1);
pcolor(tau,W,Isim);
shading flat;
xlabel('Delay (fs)','Fontsize',16)
ylabel('Energy (eV)','Fontsize',16)
title('Experimental Spectrogram','Fontsize',16);
colormap(gca,tmap)
set(gcf,'Color',[1 1 1])
set(gca,'Fontsize',14)
ylim(Yrange)

% inizializing
Irec = zeros(fn,taun);   % reconstructed trace
iter = 1:verb:noi+1;     % iteration vector
err = zeros(size(iter)); % FROG error
i_err = 1;               % needed for error
gamma_ip = 1;            % error minimization

% inital guess XUV and IR
Ptrec = exp(-A0*1i*exp(-(t/4).^2).*cos(wp*t));   % initial guess probe field
Rtrec = exp(-(t/1).^2+1i*rand(1,fn));            % initial guess atto pulse

%% loop
for ii = 1:noi
    for ij = 1:taun
        % spectrum for actual time delay
        Iw = Isim(:,ij).';
        
        % store old probe and object
        Rtold = Rtrec;
        Ptold = Ptrec;
        
        % UPDATE OBJECT
        % time delayed probe pulse
        Pwold = fftshift(fft(fftshift(Ptold)));
        Ptshift = fftshift(ifft(fftshift(Pwold.*exp(1i*w*tau(ij)))));
        
        % Fourier transform of product field
        Gtold = Ptshift.*Rtold;
        Gwold = fftshift(fft(fftshift(Gtold)));
        
        % replace modulus with measurement
        Gwnew = sqrt(Iw).*exp(1i*angle(Gwold));
        
        % inverse Fourier transformation
        Gtnew = fftshift(ifft(fftshift(Gwnew)));
        
        % difference
        DeltaG = Gtnew-Gtold;
        
        % update object function
        Rupdate = conj(Ptshift)./max(abs(Ptshift).^2);
        Rtrec = Rtold+beta_R*Rupdate.*DeltaG;
        
        % UPDATE PROBE PULSE
        % time delayed object
        Rwold = fftshift(fft(fftshift(Rtold)));
        Rtshift = fftshift(ifft(fftshift(Rwold.*exp(-1i*w*tau(ij)))));
        
        % Fourier transform of product field
        Gtold = Ptold.*Rtshift;
        Gwold = fftshift(fft(fftshift(Gtold)));
        
        % replace modulus with measurement
        Gwnew = sqrt(Iw).*exp(1i*angle(Gwold));
        
        % inverse Fourier transformation
        Gtnew = fftshift(ifft(fftshift(Gwnew)));
        
        % difference
        DeltaG = Gtnew-Gtold;
        
        % update probe pulse
        Pupdate = conj(Rtshift)./max(abs(Rtshift).^2);
        Ptrec = Ptold+beta_P*Pupdate.*DeltaG;
        Ptrec = exp(1i*angle(Ptrec));
    end
    if (ii==1||mod(ii,verb)==0)
        
        for iii = 1:taun
            Pwrec = fftshift(fft(fftshift(Ptrec)));
            h1 = fftshift(ifft(fftshift(Pwrec.*exp(1i*w*tau(iii)))));
            ht = h1.*Rtrec;
            hw = fftshift(fft(fftshift(ht)));
            Irec(:,iii) = abs(hw).^2;
        end;
        Irec = Irec/max(max(abs(Irec)));
        
        % rms error between reconstructed and measured distribution
        fun = @(gamma) sqrt(sum(sum(abs(Irec-gamma*Isim).^2))/(fn*taun));
        [gamma_min,err(i_err)] = fminsearch(fun,gamma_ip); % finding error minimum
        i_err = i_err+1; % updating index
        
        % plotting pulse and gate
        subplot(3,2,3);
        plot(t,Rt_abs,'-k');
        hold on
        plot(t,abs(Rtrec)/max(abs(Rtrec)),'--or');
        hold off
        legend('|E_{XUV}(t)|',2);
        ylabel('PULSE','Fontsize',16);
        xlabel('Time (fs)','Fontsize',16);
        xlim(Xrange)
        set(gca,'Fontsize',14)
        
        subplot(3,2,4);
        plot(t,unwrap(angle(Ptrec)),'r');
        legend('Gate(t)',2);
        ylabel('GATE','Fontsize',16);
        xlabel('Time (fs)','Fontsize',16);
        xlim([-10 10])
        set(gca,'Fontsize',14)
        
        subplot(3,2,5);
        plot(t,Rt_ph,'-k');
        hold on
        plot(t,angle(Rtrec),'--ob');
        hold off
        legend('angle(E_{XUV}(t))',2);
        ylabel('PULSE','Fontsize',16);
        xlabel('Time (fs)','Fontsize',16);
        xlim(Xrange)
        set(gca,'Fontsize',14)
        
        subplot(3,2,6);
        loglog(iter,err,'-.ok')
        xlabel('#iteration','FontSize',16)
        ylabel('Error','FontSize',16)
        legend('{\surdE_{\gamma}}')
        set(gca,'Ygrid','on')
        set(gca,'Xgrid','on')
        set(gca,'Fontsize',14)
        
        subplot(3,2,2);
        pcolor(tau,W,Irec);
        shading flat;
        xlabel('Delay (fs)','Fontsize',16)
        ylabel('Energy (eV)','Fontsize',16)
        title(['Iteration: ' num2str(ii) '    FROG error: ' num2str(err(i_err-1),'%3.3e')],'Fontsize',16);
        colormap(gca,tmap)
        set(gca,'Fontsize',14)
        ylim(Yrange)
        drawnow
    end;
end;

%% IR field
n_smooth = 10; % smoothing necessary after the numerical derivative

% extracting the IR field from the gate
dt_au = (t(2)-t(1))*1e-15/2.418884326505e-17;                   % time interval in atomic units
E0_eV = sum(mean(Isim(:,1:5),2).'.*W)/sum(mean(Isim(:,1:5),2)); % central energy
p0_au = sqrt(2*E0_eV/27.711);                                   % central momentum in atomic units
% vector potential
A = [0 -p0_au + sqrt(p0_au^2 + 2*diff(smooth(unwrap(-angle(Ptrec)),n_smooth).')/dt_au)];
t1 = t*1e-15 - 0.5*mean(abs(diff(t*1e-15)));                    % new time axis
A_sm = smooth(real(A)/max(real(A)),n_smooth);
% electric field
E_rec = -diff(A_sm);
t2 = (t1(2:end)+t1(1:end-1))*0.5;                               % new time axis
E_sm = smooth(E_rec,n_smooth);                                  % smoothing

figure
plot(t,IR_abs,'-k');
hold on
plot(t2/1e-15,E_sm/max(E_sm(5:end-5)),'--or')
set(gca,'FontSize',14)
set(gcf,'Color',[1 1 1])
xlim([-10 10])
xlabel('Time (fs)','FontSize',16);
ylabel('Norm. Amplitude','FontSize',16);
legend('Sim','Reconstructed')